function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { createContext, Children } from "react";
import Portal from "@reach/portal";
import Rect from "@reach/rect";
import WindowSize from "@reach/window-size";
import Component from "@reach/component-component";
import { node, func, object, string, number, oneOfType, any } from "prop-types";
import { wrapEvent, checkStyles, assignRef } from "@reach/utils";

var noop = function noop() {}; // TODO: add the mousedown/drag/mouseup to select of native menus, will
// also help w/ remove the menu button tooltip hide-flash.
// TODO: add type-to-highlight like native menus


var _createContext = createContext(),
    Provider = _createContext.Provider,
    Consumer = _createContext.Consumer;

var checkIfAppManagedFocus = function checkIfAppManagedFocus(_ref2) {
  var refs = _ref2.refs,
      state = _ref2.state,
      prevState = _ref2.prevState;

  if (!state.isOpen && prevState.isOpen) {
    return !refs.menu.contains(document.activeElement);
  }

  return false;
};

var manageFocusOnUpdate = function manageFocusOnUpdate(_ref3, appManagedFocus) {
  var refs = _ref3.refs,
      state = _ref3.state,
      prevState = _ref3.prevState;

  if (state.isOpen && !prevState.isOpen) {
    window.__REACH_DISABLE_TOOLTIPS = true;

    if (state.selectionIndex !== -1) {
      // haven't measured the popover yet, give it a frame otherwise
      // we'll scroll to the bottom of the page >.<
      requestAnimationFrame(function () {
        if (refs.items[state.selectionIndex]) {
          refs.items[state.selectionIndex].focus();
        }
      });
    } else {
      refs.menu.focus();
    }
  } else if (!state.isOpen && prevState.isOpen) {
    if (!appManagedFocus) {
      refs.button.focus();
    } // we want to ignore the immediate focus of a tooltip so it doesn't pop
    // up again when the menu closes, only pops up when focus returns again
    // to the tooltip (like native OS tooltips)


    window.__REACH_DISABLE_TOOLTIPS = false;
  } else if (state.selectionIndex !== prevState.selectionIndex) {
    if (state.selectionIndex === -1) {
      // clear highlight when mousing over non-menu items, but focus the menu
      // so the the keyboard will work after a mouseover
      refs.menu.focus();
    } else {
      refs.items[state.selectionIndex].focus();
    }
  }
};

var openAtFirstItem = function openAtFirstItem(state) {
  return {
    isOpen: true,
    selectionIndex: 0
  };
};

var close = function close(state) {
  return {
    isOpen: false,
    selectionIndex: -1,
    closingWithClick: false
  };
};

var selectItemAtIndex = function selectItemAtIndex(index) {
  return function (state) {
    return {
      selectionIndex: index
    };
  };
};

var genId = function genId(prefix) {
  return prefix + "-" + Math.random().toString(32).substr(2, 8);
}; ////////////////////////////////////////////////////////////////////////


var getMenuRefs = function getMenuRefs() {
  return {
    button: null,
    menu: null,
    items: []
  };
};

var getInitialMenuState = function getInitialMenuState() {
  return {
    isOpen: false,
    buttonRect: undefined,
    selectionIndex: -1,
    closingWithClick: false,
    buttonId: genId("button")
  };
};

var checkIfStylesIncluded = function checkIfStylesIncluded() {
  return checkStyles("menu-button");
};

var Menu = function Menu(_ref4) {
  var children = _ref4.children;
  return React.createElement(Component, {
    getRefs: getMenuRefs,
    getInitialState: getInitialMenuState,
    didMount: checkIfStylesIncluded,
    didUpdate: manageFocusOnUpdate,
    getSnapshotBeforeUpdate: checkIfAppManagedFocus
  }, function (context) {
    return React.createElement(Provider, {
      value: context
    }, typeof children === "function" ? children({
      isOpen: context.state.isOpen
    }) : children);
  });
};

if (process.env.NODE_ENV !== "production") {
  Menu.propTypes = {
    children: oneOfType([func, node])
  };
} ////////////////////////////////////////////////////////////////////////


var MenuButton = React.forwardRef(function (_ref5, _ref8) {
  var onClick = _ref5.onClick,
      onKeyDown = _ref5.onKeyDown,
      onMouseDown = _ref5.onMouseDown,
      props = _objectWithoutPropertiesLoose(_ref5, ["onClick", "onKeyDown", "onMouseDown"]);

  return React.createElement(Consumer, null, function (_ref6) {
    var refs = _ref6.refs,
        state = _ref6.state,
        setState = _ref6.setState;
    return React.createElement(Rect, {
      observe: state.isOpen,
      onChange: function onChange(buttonRect) {
        return setState({
          buttonRect: buttonRect
        });
      }
    }, function (_ref7) {
      var rectRef = _ref7.ref;
      return React.createElement("button", _extends({
        id: state.buttonId,
        "aria-haspopup": "menu",
        "aria-expanded": state.isOpen,
        "data-reach-menu-button": true,
        type: "button",
        ref: function ref(node) {
          rectRef(node);
          assignRef(_ref8, node);
          refs.button = node;
        },
        onMouseDown: wrapEvent(onMouseDown, function () {
          if (state.isOpen) {
            setState({
              closingWithClick: true
            });
          }
        }),
        onClick: wrapEvent(onClick, function () {
          if (state.isOpen) {
            setState(close);
          } else {
            setState(openAtFirstItem);
          }
        }),
        onKeyDown: wrapEvent(onKeyDown, function (event) {
          if (event.key === "ArrowDown") {
            event.preventDefault(); // prevent scroll

            setState(openAtFirstItem);
          } else if (event.key === "ArrowUp") {
            event.preventDefault(); // prevent scroll

            setState(openAtFirstItem);
          }
        })
      }, props));
    });
  });
});

if (process.env.NODE_ENV !== "production") {
  MenuButton.propTypes = {
    onClick: func,
    onKeyDown: func,
    children: node
  };
}

var MenuItem = React.forwardRef(function (_ref9, _ref10) {
  var onSelect = _ref9.onSelect,
      onClick = _ref9.onClick,
      _ref9$role = _ref9.role,
      role = _ref9$role === void 0 ? "menuitem" : _ref9$role,
      state = _ref9.state,
      setState = _ref9.setState,
      index = _ref9.index,
      onKeyDown = _ref9.onKeyDown,
      onMouseMove = _ref9.onMouseMove,
      onMouseLeave = _ref9.onMouseLeave,
      _ref = _ref9._ref,
      rest = _objectWithoutPropertiesLoose(_ref9, ["onSelect", "onClick", "role", "state", "setState", "index", "onKeyDown", "onMouseMove", "onMouseLeave", "_ref"]);

  var isSelected = index === state.selectionIndex;

  var select = function select() {
    onSelect();
    setState(close);
  };

  return React.createElement("div", _extends({}, rest, {
    ref: function ref(node) {
      assignRef(_ref10, node);
      assignRef(_ref, node);
    },
    "data-reach-menu-item": role === "menuitem" ? true : undefined,
    role: role,
    tabIndex: "-1",
    "data-selected": role === "menuitem" && isSelected ? true : undefined,
    onClick: wrapEvent(onClick, function (event) {
      select();
    }),
    onKeyDown: wrapEvent(onKeyDown, function (event) {
      if (event.key === "Enter" || event.key === " ") {
        // prevent the button from being "clicked" by
        // this "Enter" keydown
        event.preventDefault();
        select();
      }
    }),
    onMouseMove: wrapEvent(onMouseMove, function (event) {
      if (!isSelected) {
        setState(selectItemAtIndex(index));
      }
    }),
    onMouseLeave: wrapEvent(onMouseLeave, function (event) {
      // clear out selection when mouse over a non-menu item child
      setState({
        selectionIndex: -1
      });
    })
  }));
});

if (process.env.NODE_ENV !== "production") {
  MenuItem.propTypes = {
    onSelect: func.isRequired,
    onClick: func,
    role: string,
    state: object,
    setState: func,
    index: number,
    onKeyDown: func,
    onMouseMove: func,
    _ref: func
  };
} ////////////////////////////////////////////////////////////////////////


var MenuLink = React.forwardRef(function (_ref11, _ref12) {
  var onKeyDown = _ref11.onKeyDown,
      onClick = _ref11.onClick,
      Comp = _ref11.component,
      _ref11$as = _ref11.as,
      AsComp = _ref11$as === void 0 ? "a" : _ref11$as,
      style = _ref11.style,
      setState = _ref11.setState,
      state = _ref11.state,
      index = _ref11.index,
      _ref = _ref11._ref,
      props = _objectWithoutPropertiesLoose(_ref11, ["onKeyDown", "onClick", "component", "as", "style", "setState", "state", "index", "_ref"]);

  var Link = Comp || AsComp;

  if (Comp) {
    console.warn("[@reach/menu-button]: Please use the `as` prop instead of `component`.");
  }

  return React.createElement(MenuItem, {
    role: "none",
    state: state,
    setState: setState,
    index: index,
    onSelect: noop,
    _ref: noop
  }, React.createElement(Link, _extends({
    role: "menuitem",
    "data-reach-menu-item": true,
    tabIndex: "-1",
    "data-selected": index === state.selectionIndex ? true : undefined,
    onClick: wrapEvent(onClick, function (event) {
      setState(close);
    }),
    onKeyDown: wrapEvent(onKeyDown, function (event) {
      if (event.key === "Enter") {
        // prevent MenuItem's preventDefault from firing,
        // allowing this link to work w/ the keyboard
        event.stopPropagation();
      }
    }),
    ref: function ref(node) {
      assignRef(_ref, node);
      assignRef(_ref12, node);
    },
    style: _extends({}, style)
  }, props)));
});

if (process.env.NODE_ENV !== "production") {
  MenuLink.propTypes = {
    onKeyDown: func,
    onClick: func,
    component: any,
    as: any,
    style: object,
    setState: func,
    state: object,
    index: number,
    _ref: func
  };
} ///////////////////////////////////////////////////////////////////


var MenuList = React.forwardRef(function (props, ref) {
  return React.createElement(Consumer, null, function (_ref13) {
    var refs = _ref13.refs,
        state = _ref13.state,
        setState = _ref13.setState;
    return state.isOpen && React.createElement(Portal, null, React.createElement(WindowSize, null, function () {
      return React.createElement(Rect, null, function (_ref14) {
        var menuRect = _ref14.rect,
            menuRef = _ref14.ref;
        return React.createElement("div", {
          "data-reach-menu": true,
          ref: menuRef,
          style: getStyles(state.buttonRect, menuRect)
        }, React.createElement(MenuListImpl, _extends({}, props, {
          setState: setState,
          state: state,
          refs: refs,
          ref: ref
        })));
      });
    }));
  });
});

if (process.env.NODE_ENV !== "production") {
  MenuList.propTypes = {
    children: node
  };
}

var focusableChildrenTypes = [MenuItem, MenuLink];

var isFocusableChildType = function isFocusableChildType(child) {
  return focusableChildrenTypes.includes(child.type);
};

var getFocusableMenuChildren = function getFocusableMenuChildren(childrenArray) {
  var focusable = childrenArray.filter(function (child) {
    return isFocusableChildType(child);
  });
  return focusable;
};

var MenuListImpl = React.forwardRef(function (_ref15, _ref16) {
  var refs = _ref15.refs,
      state = _ref15.state,
      setState = _ref15.setState,
      children = _ref15.children,
      onKeyDown = _ref15.onKeyDown,
      onBlur = _ref15.onBlur,
      rest = _objectWithoutPropertiesLoose(_ref15, ["refs", "state", "setState", "children", "onKeyDown", "onBlur"]);

  var clones = Children.toArray(children).filter(Boolean);
  var focusableChildren = getFocusableMenuChildren(clones);
  return React.createElement("div", _extends({
    "data-reach-menu-list": true
  }, rest, {
    role: "menu",
    "aria-labelledby": state.buttonId,
    tabIndex: "-1",
    ref: function ref(node) {
      refs.menu = node;
      assignRef(_ref16, node);
    },
    onBlur: function onBlur(event) {
      if (!state.closingWithClick && !refs.menu.contains(event.relatedTarget || document.activeElement)) {
        setState(close);
      }
    },
    onKeyDown: wrapEvent(onKeyDown, function (event) {
      if (event.key === "Escape") {
        setState(close);
      } else if (event.key === "ArrowDown") {
        event.preventDefault(); // prevent window scroll

        var nextIndex = state.selectionIndex + 1;

        if (nextIndex !== focusableChildren.length) {
          setState({
            selectionIndex: nextIndex
          });
        }
      } else if (event.key === "ArrowUp") {
        event.preventDefault(); // prevent window scroll

        var _nextIndex = state.selectionIndex - 1;

        if (_nextIndex !== -1) {
          setState({
            selectionIndex: _nextIndex
          });
        }
      } else if (event.key === "Tab") {
        event.preventDefault(); // prevent leaving
      }
    })
  }), clones.map(function (child, index) {
    if (isFocusableChildType(child)) {
      var focusIndex = focusableChildren.indexOf(child);
      return React.cloneElement(child, {
        setState: setState,
        state: state,
        index: focusIndex,
        _ref: function _ref(node) {
          return refs.items[focusIndex] = node;
        }
      });
    }

    return child;
  }));
});

if (process.env.NODE_ENV !== "production") {
  MenuListImpl.propTypes = {
    refs: object,
    state: object,
    setState: func,
    children: node,
    onKeyDown: func,
    onBlur: func
  };
}

var getStyles = function getStyles(buttonRect, menuRect) {
  var haventMeasuredButtonYet = !buttonRect;

  if (haventMeasuredButtonYet) {
    return {
      opacity: 0
    };
  }

  var haventMeasuredMenuYet = !menuRect;
  var styles = {
    left: buttonRect.left + window.pageXOffset + "px",
    top: buttonRect.top + buttonRect.height + window.pageYOffset + "px"
  };

  if (haventMeasuredMenuYet) {
    return _extends({}, styles, {
      opacity: 0
    });
  }

  if (buttonRect.width < 500) {
    styles.minWidth = buttonRect.width;
  }

  var collisions = {
    top: buttonRect.top - menuRect.height < 0,
    right: window.innerWidth < buttonRect.left + menuRect.width,
    bottom: window.innerHeight < buttonRect.top + menuRect.height,
    left: buttonRect.left - menuRect.width < 0
  };
  var directionRight = collisions.right && !collisions.left;
  var directionUp = collisions.bottom && !collisions.top;
  return _extends({}, styles, {
    left: directionRight ? buttonRect.right - menuRect.width + window.pageXOffset + "px" : buttonRect.left + window.pageXOffset + "px",
    top: directionUp ? buttonRect.top - menuRect.height + window.pageYOffset + "px" : buttonRect.top + buttonRect.height + window.pageYOffset + "px"
  });
};

export { Menu, MenuList, MenuButton, MenuLink, MenuItem };